from q2gui.pyqt6.widgets import (
    q2label,
    q2image,
    q2line,
    q2frame,
    q2check,
    q2toolbar,
    q2tab,
    q2text,
    q2code,
    q2button,
    q2relation,
    q2radio,
    q2lookup,
    q2combo,
    q2toolbutton,
    q2progressbar,
    q2grid,
    q2sheet,
    q2scroller,
    q2date,
    q2space,
    q2list,
    q2spin,
    q2doublespin,
    q2time,
    q2color,
)


(
    q2label,
    q2image,
    q2line,
    q2frame,
    q2check,
    q2toolbar,
    q2tab,
    q2text,
    q2code,
    q2button,
    q2relation,
    q2radio,
    q2lookup,
    q2combo,
    q2toolbutton,
    q2progressbar,
    q2grid,
    q2sheet,
    q2scroller,
    q2date,
    q2space,
    q2list,
    q2spin,
    q2doublespin,
    q2time,
    q2color,
)
